/*****************************************************************************\
**                                                                           **
** BEEP-BEEP                                                                 **
**                                                                           **
**---------------------------------------------------------------------------**
** Copyright: Andreas Eversberg                                              **
**                                                                           **
** minimal blueboxing software                                               **
**                                                                           **
** 06 dec 2021 polprog - port to ALSA, code cleanup                          **
**                                                                           **
\*****************************************************************************/


#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <linux/soundcard.h>
#include <alsa/asoundlib.h>

#define SAMPLE_RATE	8000
#define DSP_DEVICE      "default"
#define PI		3.14159265

struct ccitt5 {
	char digit;
	int f1, f2;
	int duration, delay;
} ccitt5[] = {
	{'A', 2400,    0, 150, 100},
	{'B', 2600,    0, 850, 100},
	{'C', 2400, 2600,  70, 100},
	{'1',  700,  900,  55,  50},
	{'2',  700, 1100,  55,  50},
	{'3',  900, 1100,  55,  50},
	{'4',  700, 1300,  55,  50},
	{'5',  900, 1300,  55,  50},
	{'6', 1100, 1300,  55,  50},
	{'7',  700, 1500,  55,  50},
	{'8',  900, 1500,  55,  50},
	{'9', 1100, 1500,  55,  50},
	{'0', 1300, 1500,  55,  50},
	{'d',  700, 1700,  55,  50},
	{'e',  900, 1700,  55,  50},
	{'a', 1100, 1700, 100,  50},
	{'b', 1300, 1700, 100,  50},
	{'c', 1500, 1700,  55,  50},
	{0, 0, 0, 0, 0}
};

/*
 * stream bell sound
 */
void dial_dsp(snd_pcm_t *handle, char *dial)
{
	int i, j, len;
	double k1, k2;
	unsigned char buffer[SAMPLE_RATE]; /* one second */
	int frames;

	snd_pcm_prepare(handle);
	while(*dial) {
		i = 0;
		while(ccitt5[i].digit) {
			if (ccitt5[i].digit == *dial)
				break;
			i++;
		}
		if (ccitt5[i].digit) {
			/* send tone */
			len = ccitt5[i].duration * SAMPLE_RATE / 1000;
			k1 = 2.0 * PI * ccitt5[i].f1 / SAMPLE_RATE;
			k2 = 2.0 * PI * ccitt5[i].f2 / SAMPLE_RATE;
			for (j = 0; j < len; j++)
				buffer[j] = 64.0 * (2.0 + sin(k1 * j) + sin(k2 * j));
			frames = snd_pcm_writei(handle, buffer, len);

			if (frames < 0)
			  frames = snd_pcm_recover(handle, frames, 0);
			if (frames < 0)
			  printf("snd_pcm_writei failed: %s\n", snd_strerror(frames));
			/* send pause */
			len = ccitt5[i].delay * SAMPLE_RATE / 1000;
			memset(buffer, 128, len);
			frames = snd_pcm_writei(handle, buffer, len);
			if (frames < 0)
			frames = snd_pcm_recover(handle, frames, 0);
			if (frames < 0)
			  printf("snd_pcm_writei failed: %s\n", snd_strerror(frames));			
		}
		dial++;
	}
	int err = snd_pcm_drain(handle);
	if (err < 0)
		printf("snd_pcm_drain failed: %s\n", snd_strerror(err));
}


/*
 * open dsp
 */
snd_pcm_t *open_dsp(char *device, int ratem, snd_pcm_t *handle)
{
	int err;
	if ((err = snd_pcm_open(&handle, device, SND_PCM_STREAM_PLAYBACK,
				0)) < 0) {
	printf("Playback open error: %s\n", snd_strerror(err));
	exit(EXIT_FAILURE);
	}
	if ((err = snd_pcm_set_params(handle,
				  SND_PCM_FORMAT_U8,
				  SND_PCM_ACCESS_RW_INTERLEAVED,
				  1,
				  8000,
				  1,
				  500000)) < 0) {   /* 0.5sec */
		printf("Playback open error: %s\n", snd_strerror(err));
		exit(EXIT_FAILURE);
	}
	return handle;
}

void close_dsp(snd_pcm_t *handle)
{
	int err = snd_pcm_close(handle);
	if(err < 0){
	printf("Playback close error: %s\n", snd_strerror(err));
		exit(EXIT_FAILURE);
	}
}

int main(int argc, char *argv[])
{
	snd_pcm_t *handle;
	char *dial = "";
	int ch;
	char digit[2] = "x";
	struct termios orig, now;

	if (argc > 1 && !strcmp(argv[1], "--help")) {
		printf("Usage: %s [<dial string>]\n\n", argv[0]);
		printf("Runs with with the given dial string.\n");
		printf("For KP1, KP2, ST, use characters a, b, c instead.\n");
		return 0;
	}

	if (argc > 1)
		dial = argv[1];

	handle = open_dsp(DSP_DEVICE, SAMPLE_RATE, handle);

	printf("Welcome to Beep-Beep, your simple bluebox!\n");
	printf("------------------------------------------\n");
	printf("y or z = clear-forward\n");
	printf("x = seize\n");
	printf("0-9 = Digits\n");
	printf("a = KP1\n");
	printf("b = KP2\n");
	printf("c = ST\n");
	printf("d = Code 11\n");
	printf("e = Code 12\n");
	printf("f = forward transfer\n");
	if (dial[0])
		printf(". = dial %s\n", dial);
	printf("Escape to exit.\n");

	tcgetattr(0, &orig);
	now = orig;
	now.c_lflag &= ~(ISIG|ICANON|ECHO);
	now.c_cc[VMIN]=1;
	now.c_cc[VTIME]=2;

	tcsetattr(0, TCSANOW, &now);
	while ((ch = getc(stdin)) != 27) {
		switch(ch) {
		case 'y': case 'z':
			dial_dsp(handle, "C");
			break;
		case 'x':
			dial_dsp(handle, "A");
			break;
		case 'f':
			dial_dsp(handle, "B");
			break;
		case '1': case '2': case '3': case '4': case '5':
		case '6': case '7': case '8': case '9': case '0':
		case 'a': case 'b': case 'c': case 'd': case 'e':
			digit[0] = ch;
			dial_dsp(handle, digit);
			break;
		case '.':
			if (dial[0])
				dial_dsp(handle, dial);
			break;
		}
		if (ch == 3)
			break;
	}
	tcsetattr(0, TCSANOW, &orig);

	close_dsp(handle);

	return 0;
}


